#!/usr/bin/env python
"""
This module provides very simple dynamic filebrowser.
Its main purpose is to show how to build dynamic trees that don't
have to store the full tree all at once.
"""

# Don't do 'from gtk import *'.  This module needs only a few
# gtk features, and shouldn't fatten its namespace w. gtk symbols.
import gtk, GTK
import os

class FileBrowser:
    def __init__(self, directory="."):
        """Initialize the file browser from a given directory"""
        self.widget = gtk.GtkHBox()
        self.tree = gtk.GtkCTree( cols=2, titles=["Directory", "File"] )


        # keep all the unexpanded paths stored in a dictionary
        self.unexpanded_paths = {}
        # add the base file to the tree
        self._add_file(directory)
        
        self.tree.show()
        self.tree.set_selection_mode(gtk.SELECTION_MULTIPLE)
        # make a window
        self.scrolled_win = gtk.GtkScrolledWindow()
        self.scrolled_win.set_policy(gtk.POLICY_AUTOMATIC,
                                     gtk.POLICY_AUTOMATIC)
        # size the window and pack
        self.scrolled_win.set_usize(200, 200)
        self.scrolled_win.show()
        self.widget.pack_start(self.scrolled_win,
                               expand=gtk.TRUE, fill=gtk.TRUE)
        self.scrolled_win.add(self.tree)

        self.tree.connect("tree-expand", self.tree_expand)

    def show(self):
        """show the FileBrowser"""
        self.widget.show()
        
    def hide(self):
        """hide the FileBrowser"""
        self.widget.hide()
        
    def delete(self):
        """Delete the file browser"""
        # well, hide it actually
        self.widget.hide()
        
    def tree_expand(self, tree, node):
        """catch the tree-expand signal, if the directory has not
        been updated, read all of the files in the directory and
        add them to the tree.  Remove the directory from the list
        of unexpanded directories and remove the bookeeping node"""
        try:
            self.tree.freeze()
            path = self.tree.node_get_row_data(node)
            if path in self.unexpanded_paths.keys():
                # add the files in the directory
                expand_node, bookkeeping_node = self.unexpanded_paths[path]
                for file in os.listdir(path):
                    self._add_file(os.path.join(path, file),
                                   expand_node)
                # remove the path from the list of unexpanded paths
                del self.unexpanded_paths[path]
                # remove the bookeeping node
                self.tree.remove_node(bookkeeping_node)
            # resize the columns
            self.tree.columns_autosize()
        finally:
            self.tree.thaw()

    def _add_file(self, path, parent=None):
        """Add a file/directory to the tree as a child of parent.
        If parent is None, then add to the root of the tree"""
        assert os.path.exists(path), "Whups, path doesn't exist"

        file = os.path.split(path)[-1]
        if os.path.isdir(path):
            is_leaf = 0
        else:
            is_leaf = 1
            
        node=self.tree.insert_node(parent, None,
                                   text = [path, file],
                                   is_leaf = is_leaf,
                                   expanded = gtk.FALSE)
        
        if not is_leaf:
            # add the bookeeping node
            bookkeep_node = self.tree.insert_node(node, None,
                                                  text=["book", "node"],
                                                  is_leaf = gtk.TRUE,
                                                  expanded = gtk.FALSE)
            # cache the path
            self.unexpanded_paths[path] = (node, bookkeep_node)
            # add the path to the node data so we can look it up
            self.tree.node_set_row_data(node, path)

            
def main():
    """Module mainline (for standalone execution)"""
    window = gtk.GtkWindow()
    window.connect("delete_event", gtk.mainquit)
    t = FileBrowser(".")
    window.add(t.widget)
    t.show()
    window.show()
    gtk.mainloop()


if __name__ == "__main__":
    main()
