#!/usr/bin/env python
"""
This module provides a sort of 'augmented' GtkCTree.
The tree provides additional behaviors such as column
addition, column popup menus, etc.
"""
# Don't do 'from gtk import *'.  This module needs only a few
# gtk features, and shouldn't fatten its namespace w. gtk symbols.
import gtk, GTK, _gtk
from ColumnController import ColumnController

class CTreeInsert(gtk.GtkCTree):
    """This is the augmented GtkCTree class.
    See the module docstring for more info.
    -- I'm kind of uneasy about subclassing from a GtkCTree, rather
    than composing on top of it, but... well, there it is."""
    # make 100 columns available
    _available_columns = 100

    def __init__(self, cols=1, titles=["Column 1"]):
        """Initialize a new instance."""
        # we need to initialize the tree with all of the available
        # columns even though we will hide most of these
        # if we don't have enough space, make enough space.
        # we won't be able to add columns though
        if self._available_columns < cols:
            self._available_columns = cols
        used_titles = map(str, range(self._available_columns))
        for i in range(len(titles)):
            used_titles[i] = titles[i]

        gtk.GtkCTree.__init__(self, cols=self._available_columns,
                              titles=used_titles)
        # keep the number of columns actually used
        self.used_columns = cols

        # Cache the column button widgets.  GtkClist makes it too
        # painful to get at them otherwise.
        self.column_buttons = []
        for i in range(self._available_columns):
            self.column_buttons.append(self.get_column_widget(i)["parent"])

        self.column_controller = ColumnController(self)
        self._init_column_visibility()

        # This fixes TreeViewer Task 121, but I don't know why.
        # If you wait till all nodes have been added before auto-sizing
        # you get bad results.
        self.columns_autosize()

    def insert_node(self, parent, sibling, text, spacing=5,
                    pixmap_closed=None, mask_closed=None,
                    pixmap_opened=None, mask_opened=None, is_leaf=gtk.TRUE,
                    expanded=gtk.FALSE):
        # We need to overwrite insert_node, since the number of
        # columns that are actually in the tree are potentially more
        # than the user thinks.
        full_text = text + [""] * (self._available_columns - len(text))
        
        # parent and sibling can also be None
        return _gtk.gtk_ctree_insert_node(self._o, parent, sibling,
                                          full_text, spacing,
                                          pixmap_closed, mask_closed,
                                          pixmap_opened, mask_opened,
                                          is_leaf, expanded)

    def has_columns(self):
        """return 1 if there are columns available, 0 if not"""
        return (self.used_columns < self._available_columns)
    
    def column_in_use(self, i):
        """Find out whether or not column i is in use.
        Override this method if you need anything more sophisticated."""
        # Assume that any column having a non-empty title
        # is in use.
        return (i < self.used_columns)

    def columns_autosize(self):
        """Extend the base-class implementation to take into account
        the widths of unadded nodes."""
        self.freeze()

        try:
            # Remember which nodes are unexpanded, so we
            # can restore them to their unexpanded state after resizing.
            unexpanded_nodes = []
            all_nodes = self.all_nodes()
            for node in all_nodes:
                if not node.expanded:
                    unexpanded_nodes.append(node)

            # Make all nodes visible.  When the base class resizes
            # columns, it computes the new sizes based on the widths of
            # the visible nodes.
            for node in self.base_nodes():
                self.expand_recursive(node)

            gtk.GtkCTree.columns_autosize(self)

            # Re-hide the hidden nodes.
            for node in unexpanded_nodes:
                self.collapse(node)
        finally:
            # even if something goes wrong, unthaw the tree.
            # This way the user won't be stuck with a blank
            # screen.
            self.thaw()

    def all_nodes(self):
        """Get a list of all nodes currently in the tree."""
        result = self.base_nodes()
        index = 0
        while 1:
            new_children = []
            next_index = len(result)
            for node in result[index:]:
                new_children = new_children + node.children
            if not new_children:
                break
            result = result + new_children
            index = next_index
        
        return result

    def _init_column_visibility(self):
        """Make all 'in-use' columns visible, and hide all others."""
        for i in range(len(self.column_buttons)):
            self.set_column_visibility(i, self.column_in_use(i))

    def make_selected_nodes_visible(self):
        """Make all selected nodes in the tree visible."""
        for node in self.selection:
            parent = node.parent
            while not self.is_viewable(node) and parent:
                self.expand(parent)
                parent = parent.parent

    def node_texts(self, node):
        """Get the text for every in-use column of the specified node."""
        # James Henstridge recommends using get_node_info for getting
        # the text of the first tree column.
        firstColumnInfo = self.get_node_info(node)
        result = [firstColumnInfo[0]]
        for i in range(1, self["n_columns"]):
            if self.column_in_use(i):
                result.append(self.node_get_text(node, i))
        return result

def main():
    """Module mainline (for standalone execution)"""
    window = gtk.GtkWindow()
    window.connect("delete_event", gtk.mainquit)
    titles = ["A", "", "C", "D"]
    t = CTreeInsert(cols=len(titles), titles=titles)
    t.set_selection_mode(gtk.SELECTION_MULTIPLE)
    
    window.add(t)

    def fill_tree(t):
        """Fill the tree with some bogus content."""
        import whrandom

        def rnd(colTitle, basename, whrandom=whrandom):
            return "%s%s.%s" % (colTitle, whrandom.random(), basename)
        
        root = t.insert_node(None, None, ["Root", "", "C", "D"], is_leaf=0)
        nodes = [root]
        start_index = 0
        max_depth = 2
        for depth in range(max_depth):
            next_start = len(nodes)
            i_parent = -1
            for parent in nodes[start_index:]:
                i_parent = i_parent + 1
                for breadth in range(10):
                    name = "%s.%s.%s" % (depth, i_parent, breadth)
                    title = [rnd("A", name), "", rnd("C", name),
                             rnd("D", name)]
                    node = t.insert_node(parent, None, title,
                                         is_leaf=(depth >= (max_depth-1)))

                    # Randomly select some fraction of the nodes:
                    if whrandom.random() > 0.7:
                        t.select(node)
                        
                    nodes.append(node)
            start_index = next_start

        # Test node_texts() by invoking it on the last node.
        print "Last node:", t.node_texts(node)

    fill_tree(t)
    t.columns_autosize()
    # How to test this?  Dunno; just expand the tree and if
    # any selected nodes show up, the method didn't work.
    t.make_selected_nodes_visible()
    t.show()

    num_nodes = len(t.all_nodes())
    window.set_title("Test: %d Nodes" % (num_nodes))
    window.show()

    # This should print 'Column 1 is not used.'
    for i in range(len(titles)):
        if not t.column_in_use(i):
            print "Column %s is not used." % i

    gtk.mainloop()


if __name__ == "__main__":
    main()
