#!/usr/bin/env python
"""
This module provides popup menus for MeshTrix column headers.
Copyright 1999 Bioreason, Inc.
$Id: ColumnPopupMenu.py,v 1.6 1999/07/26 21:04:39 chapman Exp $
Mitch Chapman
"""

__version__ = "$Revision: 1.6 $"

import gtk, GDK, GtkExtra, string

# For errors raised by this module.
class Error(Exception): pass

class ColumnPopupMenu:
    """This class manages a popup menu for a MeshTrix column header.
    Note that this class can be used only with a GtkCList subclass which
    provides a column_in_use() method.
    
    Callbacks are provided for each of the provided menu items.  They
    have the following semantics:
        set_column_visibility_cb(widget, i_src, show)
           Show or hide a column.
           i_src is the index of the column to show.
           if show evaluates to true, show the column, else hide it.
        sort_cb(widget, i_column)
           Sort the MeshTrix contents on the specified i_column.
        """
    def __init__(self, tree, set_column_visibility_cb=None,
                 sort_cb=None, insert_cb=None):
        """Initialize a new instance."""
        self.tree = tree
        self.num_columns = self.tree["n_columns"]
        
        self.set_column_visibility_cb = set_column_visibility_cb
        self.sort_cb = sort_cb
        self.insert_cb = insert_cb

        self.menu = None
        self.factory = None
        self._build_menu()
        
        # During menu activation, this is the column over which
        # the menu is posted.
        self.current_column = None

    def _build_menu(self):
        """Build the actual menu contents."""
        factory = self.factory = \
                  GtkExtra.MenuFactory(type=GtkExtra.MENU_FACTORY_MENU)


        menu_item = factory.create("/Insert", None, self.insert_cb)
        # if there are no columns available for insert, grey out
        # the menu
        if not self.tree.has_columns():
            menu_item.set_sensitive(0)
            
        factory.create("/Sort", None, self.sort_current_column_cb)

        # Finally, return the popup.
        self.menu = factory.get_menu('')

    def popup(self, event, i_column):
        """Display self.
        `event' is the event triggering the popup.
        `i_column' is the column over which the menu is being popped up."""
        self.update_menu_items(i_column)
        self.current_column = i_column
        self.menu.popup(None, None, None, event.button, event.time)

    def update_menu_items(self, i_column):
        """Update menu appearance (sensitivity, etc.) based on the
        current context.
        `i_column' is the column over which the item is appearing."""
        pass

    def column_title(self, index):
        """Get the title for the specified column."""
        result = self.tree.get_column_title(index)
        # The title may contain newlines.  Replace these with simple
        # whitespace, otherwise the menu is too hard to read.
        result = string.replace(result, "\n", " ")
        return result


    def sort_current_column_cb(self, widget, *args):
        """Tell the client to sort on self's current column."""
        if self.sort_cb:
            self.sort_cb(widget, self.current_column)


def main():
    """Module mainline (for standalone execution)"""
    # This is by no means a comprehensive test.
    class Tree(gtk.GtkCTree):
        def column_in_use(self, i):
            title = self.get_column_title(i)
            return (title != "")

        def has_columns(self):
            # no available columns
            return 0
        
        def connect_menus(self):
            self.menu = ColumnPopupMenu(self,
                          set_column_visibility_cb=self.set_column_visibility_cb,
                          sort_cb=self.sort_cb)
            num_columns = self["n_columns"]
            for i in range(num_columns):
                button = self.get_column_widget(i)["parent"]
                button.connect("button_press_event", self.button_clicked_cb, i)

        def button_clicked_cb(self, widget, event, i):
            self.menu.popup(event, i)

        def set_column_visibility_cb(self, widget, i_column, show):
            self.set_column_visibility(i_column, show)

        def move_column_cb(self, widget, i_src, i_dest):
            print "Pretend column %d has moved to %d." % (i_src, i_dest)

        def sort_cb(self, widget, i):
            self.set_sort_column(i)
            self.sort()

        def fill_tree(self):
            """Fill self w. random data.
            I've used variations on this routine in so many places,
            it ought to be shoved into a separate, regression-test-support
            module.
            XXX FIX THIS."""
            import whrandom

            def rnd(colTitle, basename, whrandom=whrandom):
                r = int(100 * whrandom.random())
                return "%s%05s.%s" % (colTitle, r, basename)

            numColumns = self["n_columns"]
            rootTitle = numColumns * ["Root"]
            root = self.insert_node(None, None, rootTitle, is_leaf=0)
            nodes = [root]
            start_index = 0
            max_depth = 3
            for depth in range(max_depth):
                next_start = len(nodes)
                is_leaf = (depth >= (max_depth - 1))
                i_parent = -1
                for parent in nodes[start_index:]:
                    i_parent = i_parent + 1
                    for breadth in range(10):
                        name = "%02s.%04s.%02s" % (depth, i_parent, breadth)
                        title = []
                        for c in range(numColumns):
                            title.append(rnd("%d" % c, name))
                        node = self.insert_node(parent, None, title,
                                                is_leaf=is_leaf)

                        nodes.append(node)
                start_index = next_start
            

    w = gtk.GtkWindow()
    titles = ["Col 1", "", "Col 3", "Col 4", "", "Col 5"]
    tree = Tree(cols=len(titles), titles=titles)
    tree.fill_tree()
    tree.connect_menus()
    
    w.add(tree)
    tree.show()
    w.connect("delete_event", gtk.mainquit)

    w.show()
    gtk.mainloop()
    return


if __name__ == "__main__":
    main()
